@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  gnome-glyphlist.c: Device independent rich text representation system
  *
@@ -38,6 +38,22 @@ void gnome_glyphlist_moveto_y  (GnomeGlyphList *gl, gdouble distance);
 void gnome_glyphlist_rmoveto_x (GnomeGlyphList *gl, gdouble distance);
 void gnome_glyphlist_rmoveto_y (GnomeGlyphList *gl, gdouble distance);
 
+
+GType
+gnome_glyphlist_get_type(void)
+{
+	static GType type = 0;
+	
+	if (type == 0) {
+		type = g_boxed_type_register_static
+			("GnomeGlyphList",
+			 (GBoxedCopyFunc) gnome_glyphlist_ref,
+			 (GBoxedFreeFunc) gnome_glyphlist_unref);
+	}
+
+	return type;
+}
+
 GnomeGlyphList *
 gnome_glyphlist_new (void)
 {
@@ -769,3 +785,123 @@ ggl_ensure_rule_space (GnomeGlyphList *gl, gint space)
 }
 
 
+static void
+gglrule_dump (GGLRule const *rule)
+{
+	switch ((GGLInfoType) rule->code) {
+	case GGL_POSITION:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    POSITION:: ival: %d", rule->value.ival);
+	break;
+	case GGL_MOVETOX:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    MOVETOX:: dval: %g", rule->value.dval);
+	break;
+	case GGL_MOVETOY:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    MOVETOY:: dval: %g", rule->value.dval);
+		break;
+	case GGL_RMOVETOX:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    RMOVETOX:: dval: %g", rule->value.dval);
+		break;
+	case GGL_RMOVETOY:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    RMOVETOY:: dval: %g", rule->value.dval);
+		break;
+	case GGL_ADVANCE:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    ADVANCE:: bval: %s", (rule->value.bval ? "true" : "false"));
+		break;
+	case GGL_LETTERSPACE:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    LETTERSPACE:: dval: %g", rule->value.dval);
+		break;
+	case GGL_KERNING:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    KERNING:: dval: %g", rule->value.dval);
+		break;
+	case GGL_FONT:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    FONT:: name: '%s'; size: %g",
+		       gnome_font_get_name (rule->value.font),
+		       gnome_font_get_size (rule->value.font));
+	break;
+	case GGL_COLOR:
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    COLOR:: dval: %x", rule->value.color);
+		break;
+	}
+}
+
+void
+gnome_glyphlist_dump (GnomeGlyphList const *gl)
+{
+	gint cg, cr;
+
+	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "glyphlist (%p)", gl);
+	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+	       "  refcount: %d; glyphs: %p; g_length: %d; g_size: %d;",
+	       gl->refcount, gl->glyphs, gl->g_length, gl->g_size);
+	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+	       "  rules: %p; r_length: %d; r_size: %d",
+	       gl->rules, gl->r_length, gl->r_size);
+
+	cr = 0;
+	for (cg = 0; cg < gl->g_length; cg++) {
+		/* Collect rules */
+		while ((cr < gl->r_length) &&
+		       ((gl->rules[cr].code != GGL_POSITION) ||
+			(gl->rules[cr].value.ival <= cg))) {
+			gglrule_dump (&gl->rules[cr]);
+			cr += 1;
+		}
+
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    glyph: 0x%04X", gl->glyphs[cg]);
+	}
+
+	while ((cr < gl->r_length)) {
+		gglrule_dump (&gl->rules[cr]);
+		cr += 1;
+	}
+}
+
+#include "gnome-pgl-private.h"
+void
+gnome_glyphlist_dump_pgl (GnomeGlyphList const *gl)
+{
+	static gdouble id[] = {1.0, 0.0, 0.0, 1.0, 0.0, 0.0};
+	GnomePosGlyphList *pgl;
+	gint s, g;
+
+	pgl = gnome_pgl_from_gl (gl, id, GNOME_PGL_RENDER_DEFAULT);
+
+	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "GnomePosGlyphList");
+	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+	       "  version: %d; glyphs: %p; strings: %p; num_strings: %d",
+	       pgl->version, pgl->glyphs, pgl->strings, pgl->num_strings);
+
+	for (s = 0; s < pgl->num_strings; s++) {
+		GnomePosString *ps = pgl->strings + s;
+		GnomeFont *font = gnome_rfont_get_font (ps->rfont);
+
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "    GnomePosString");
+		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		       "      start: %d; length: %d; font: '%s'; size: %g; color: %012x",
+		       ps->start, ps->length,
+		       gnome_font_get_name (font), gnome_font_get_size (font),
+		       ps->color);
+
+		for (g = ps->start; g < ps->start + ps->length; g++) {
+			g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+			       "      GnomePosGlyph:: glyph: 0x%04X; x: %g; y: %g",
+			       pgl->glyphs[g].glyph,
+			       pgl->glyphs[g].x,
+			       pgl->glyphs[g].y);
+		}
+	}
+
+	gnome_pgl_destroy (pgl);
+}
