@@ -37,13 +37,27 @@
 #include <libgnomeprint/gnome-print-meta.h>
 #include <libgnomeprint/gnome-print-transport.h>
 #include <libgnomeprint/gnome-print-multipage.h>
-#include <libgnomeprint/gnome-print-job-private.h>
+#include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprint/gnome-print-config-private.h>
 #include <libgnomeprint/gnome-print-i18n.h>
 
-typedef struct _JOBPrivate JOBPrivate;
+typedef struct _GnomePrintJobPrivate GnomePrintJobPrivate;
 
-struct _JOBPrivate {
+struct _GnomePrintJob {
+	GObject object;
+
+	GnomePrintConfig *config;
+	GnomePrintContext *meta;
+	guchar *input_file;
+
+	GnomePrintJobPrivate *priv;
+};
+
+struct _GnomePrintJobClass {
+	GObjectClass parent_class;
+};
+
+struct _GnomePrintJobPrivate {
 	/* closed flag */
 	guint closed : 1;
 
@@ -67,12 +81,13 @@ struct _JOBPrivate {
 
 enum
 {
-  PROP_0,
-  PROP_CONFIG
+	PROP_0,
+	PROP_CONFIG,
+	PROP_CONTEXT
 };
  
 
-#define GNOME_PRINT_JOB_CLOSED(m) (((JOBPrivate *) (m)->priv)->closed)
+#define GNOME_PRINT_JOB_CLOSED(m) ((m)->priv->closed)
 
 static void gnome_print_job_class_init (GnomePrintJobClass *klass);
 static void gnome_print_job_init (GnomePrintJob *job);
@@ -120,20 +135,21 @@ gnome_print_job_class_init (GnomePrintJobClass *klass)
 	
 	object_class = (GObjectClass *) klass;
 
+	object_class->finalize     = gnome_print_job_finalize;
 	object_class->set_property = gnome_print_job_set_property;
 	object_class->get_property = gnome_print_job_get_property;
 
-	g_object_class_install_property (object_class,
-                                   PROP_CONFIG,
-                                   g_param_spec_object ("config",
-                                                        _("Job Configuration"),
-                                                        _("The configuration for the print job"),
-                                                        GNOME_TYPE_PRINT_CONFIG,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-  
-	parent_class = g_type_class_peek_parent (klass);
+	g_object_class_install_property (object_class, PROP_CONFIG,
+		g_param_spec_object ("config", _("Job Configuration"),
+			_("The configuration for the print job"),
+			GNOME_TYPE_PRINT_CONFIG,
+			G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class, PROP_CONTEXT,
+		g_param_spec_object ("context", _("Context"),
+			_("The context for the print job"),
+			GNOME_TYPE_PRINT_META, G_PARAM_READWRITE));
 
-	object_class->finalize = gnome_print_job_finalize;
+	parent_class = g_type_class_peek_parent (klass);
 }
 
 static void
@@ -141,11 +157,11 @@ gnome_print_job_init (GnomePrintJob *job)
 {
 	job->config = NULL;
 
-	job->meta = NULL;
+	job->meta = g_object_new (GNOME_TYPE_PRINT_META, NULL);
 
 	job->input_file = NULL;
 
-	job->priv = g_new0 (JOBPrivate, 1);
+	job->priv = g_new0 (GnomePrintJobPrivate, 1);
 	job_clear_config_data (job);
 }
 
@@ -182,48 +198,44 @@ static void
 gnome_print_job_get_property (GObject *object, guint prop_id, GValue *value,
 	                     GParamSpec *pspec)
 {
-  GnomePrintJob *job = GNOME_PRINT_JOB (object);
+	GnomePrintJob *job = GNOME_PRINT_JOB (object);
 
-  switch (prop_id)
-    {
-    case PROP_CONFIG:
-      g_value_set_object (value, gnome_print_job_get_config (job));
-      break;
-  
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
+	switch (prop_id) {
+	case PROP_CONFIG:
+		g_value_set_object (value, job->config);
+		break;
+	case PROP_CONTEXT:
+		g_value_set_object (value, job->meta);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 static void
 gnome_print_job_set_property (GObject * object, guint prop_id,
 	                     const GValue * value, GParamSpec * pspec)
 {
-  GnomePrintJob *job = GNOME_PRINT_JOB (object);
-  GnomePrintConfig *config = NULL;
+	GnomePrintJob *job = GNOME_PRINT_JOB (object);
 
-  switch (prop_id)
-    {
-    case PROP_CONFIG:
-	    config = GNOME_PRINT_CONFIG (g_value_get_object (value));
-	    if (job->config) {
-		    gnome_print_config_unref (job->config);
-		    job->config = NULL;
-	    }
-	    
-	    if(config) {
-		    gnome_print_config_ref (config);
-		    job->config = config;
-	    } else {
-		    /* Use the default: */
-		    job->config = gnome_print_config_default ();
-	    }
-	    break;
-    default:
-	    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-	    break;
-    }
+	switch (prop_id) {
+	case PROP_CONFIG:
+		if (job->config) gnome_print_config_unref (job->config);
+		job->config = g_value_get_object (value);
+		if (job->config) gnome_print_config_ref (job->config);
+		else job->config = gnome_print_config_default ();
+		break;
+	case PROP_CONTEXT:
+		if (job->meta) g_object_unref (job->meta);
+		job->meta = g_value_get_object (value);
+		if (job->meta) g_object_ref (G_OBJECT (job->meta));
+		else job->meta = g_object_new (GNOME_TYPE_PRINT_META, NULL);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 
@@ -284,9 +296,6 @@ gnome_print_job_get_context (GnomePrintJob *job)
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), NULL);
 	g_return_val_if_fail (job->input_file == NULL, NULL);
 
-	if (job->meta == NULL)
-		job->meta = gnome_print_meta_new ();
-
 	g_object_ref (G_OBJECT (job->meta));
 
 	return job->meta;
@@ -310,7 +319,6 @@ gnome_print_job_set_file (GnomePrintJob *job, gchar *input_file)
 	g_return_if_fail (GNOME_IS_PRINT_JOB (job));
 	g_return_if_fail (input_file != NULL);
 	g_return_if_fail (job->input_file == NULL);
-	g_return_if_fail (job->meta == NULL);
 
 	job->input_file = g_strdup (input_file);
 }
@@ -329,7 +337,6 @@ gnome_print_job_set_file (GnomePrintJob *job, gchar *input_file)
 int
 gnome_print_job_get_pages (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
 	gint mp;
 
 	g_return_val_if_fail (job != NULL, 0);
@@ -341,12 +348,10 @@ gnome_print_job_get_pages (GnomePrintJob *job)
 
 	mp = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
 
-	pp = job->priv;
-
-	if (pp->num_affines <= 1)
+	if (job->priv->num_affines <= 1)
 		return mp;
 
-	return (mp + pp->num_affines - 1) / pp->num_affines;
+	return (mp + job->priv->num_affines - 1) / job->priv->num_affines;
 }
 
 /**
@@ -379,27 +384,23 @@ gnome_print_job_get_page_size_from_config (GnomePrintConfig *config, gdouble *wi
 gboolean
 gnome_print_job_get_page_size (GnomePrintJob *job, gdouble *width, gdouble *height)
 {
-	JOBPrivate *pp;
-
 	g_return_val_if_fail (job != NULL, FALSE);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), FALSE);
 	g_return_val_if_fail (width != NULL, FALSE);
 	g_return_val_if_fail (height != NULL, FALSE);
 
-	pp = job->priv;
-
 	job_update_layout_data (job);
 
-	if (pp->LY_LIST) {
+	if (job->priv->LY_LIST) {
 		if (width)
-			*width = pp->LW;
+			*width = job->priv->LW;
 		if (height)
-			*height = pp->LH;
+			*height = job->priv->LH;
 	} else {
 		if (width)
-			*width = pp->pw;
+			*width = job->priv->pw;
 		if (height)
-			*height = pp->ph;
+			*height = job->priv->ph;
 	}
 
 	return TRUE;
@@ -418,25 +419,13 @@ gnome_print_job_get_page_size (GnomePrintJob *job, gdouble *width, gdouble *heig
 gint
 gnome_print_job_close (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
-
 	g_return_val_if_fail (job != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (job->input_file == NULL, 0);
+	g_return_val_if_fail (!GNOME_PRINT_JOB_CLOSED (job), GNOME_PRINT_ERROR_UNKNOWN);
 
-	if (!job->meta)
-		job->meta = gnome_print_meta_new ();
-
-	pp = job->priv;
-
-	if (!pp->closed) {
-		pp->closed = TRUE;
-		return gnome_print_context_close (job->meta);
-	}
-
-	g_warning ("gnome_print_job_close can only be called once\n");
-
-	return GNOME_PRINT_ERROR_UNKNOWN;
+	job->priv->closed = TRUE;
+	return gnome_print_context_close (job->meta);
 }
 
 /**
@@ -514,7 +503,6 @@ metadata_printer_done:
 gint
 gnome_print_job_print (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
 	GnomePrintContext *ctx;
 	gint lpages, copies, nstacks, npages, nsheets;
 	gboolean collate, is_multipage, hwcopies;
@@ -532,9 +520,7 @@ gnome_print_job_print (GnomePrintJob *job)
 		return gnome_print_transport_print_file (transport, job->input_file);
 	}
 
-	pp = job->priv;
-
-	if (!pp->closed) {
+	if (!job->priv->closed) {
 		g_warning ("You should call gnome_print_job_close before calling\n"
 			   "gnome_print_job_print\n");
 		gnome_print_job_close (job);
@@ -557,12 +543,12 @@ gnome_print_job_print (GnomePrintJob *job)
 	/* Update the layout data */
 	is_multipage = FALSE;
 	job_update_layout_data (job);
-	if (pp->LY_LIST) {
+	if (job->priv->LY_LIST) {
 		GnomePrintContext *mp;
 		/* Find out physical page count */
-		npages = (npages + pp->num_affines - 1) / pp->num_affines;
+		npages = (npages + job->priv->num_affines - 1) / job->priv->num_affines;
 		/* Create multipage context */
-		mp = gnome_print_multipage_new (ctx, pp->LY_LIST);
+		mp = gnome_print_multipage_new (ctx, job->priv->LY_LIST);
 		g_object_unref (G_OBJECT (ctx));
 		ctx = mp;
 		is_multipage = TRUE;
@@ -600,8 +586,8 @@ gnome_print_job_print (GnomePrintJob *job)
 			for (sheet = 0; sheet < nsheets; sheet++) {
 				gint start, i;
 				/* Render physical page */
-				start = page * pp->num_affines;
-				for (i = start; (i < (start + pp->num_affines)) && (i < lpages); i++) {
+				start = page * job->priv->num_affines;
+				for (i = start; (i < (start + job->priv->num_affines)) && (i < lpages); i++) {
 					ret = gnome_print_meta_render_data_page (ctx, buf, blen, i, TRUE);
 					g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
 				}
@@ -637,7 +623,6 @@ gnome_print_job_print (GnomePrintJob *job)
 gint
 gnome_print_job_render (GnomePrintJob *job, GnomePrintContext *ctx)
 {
-	JOBPrivate *pp;
 	const guchar *data;
 	gint len;
 	gint ret;
@@ -649,17 +634,15 @@ gnome_print_job_render (GnomePrintJob *job, GnomePrintContext *ctx)
 	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (job->meta, GNOME_PRINT_ERROR_UNKNOWN);
 
-	pp = job->priv;
-
 	data = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
 	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	len = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
 	g_return_val_if_fail (len > 0, GNOME_PRINT_ERROR_UNKNOWN);
 
 	job_update_layout_data (job);
-	if (pp->LY_LIST) {
+	if (job->priv->LY_LIST) {
 		GnomePrintContext *mp;
-		mp = gnome_print_multipage_new (ctx, pp->LY_LIST);
+		mp = gnome_print_multipage_new (ctx, job->priv->LY_LIST);
 		ret = gnome_print_meta_render_data (mp, data, len);
 		gnome_print_multipage_finish_page (GNOME_PRINT_MULTIPAGE (mp));
 		g_object_unref (G_OBJECT (mp));
@@ -684,7 +667,6 @@ gnome_print_job_render (GnomePrintJob *job, GnomePrintContext *ctx)
 gint
 gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint page, gboolean pageops)
 {
-	JOBPrivate *pp;
 	const guchar *data;
 	gint len, pages;
 	gint ret;
@@ -696,8 +678,6 @@ gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint pa
 	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (job->meta, GNOME_PRINT_ERROR_UNKNOWN);
 
-	pp = job->priv;
-
 	data = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
 	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	len = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
@@ -705,17 +685,17 @@ gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint pa
 	pages = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
 
 	job_update_layout_data (job);
-	if (pp->LY_LIST) {
+	if (job->priv->LY_LIST) {
 		GnomePrintContext *meta, *mp;
 		gint start, p;
 
 		meta = gnome_print_meta_new ();
 		g_return_val_if_fail (meta != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 		/* Create wrapper multipage */
-		mp = gnome_print_multipage_new (meta, pp->LY_LIST);
-		start = page * pp->num_affines;
+		mp = gnome_print_multipage_new (meta, job->priv->LY_LIST);
+		start = page * job->priv->num_affines;
 		/* Render job pages */
-		for (p = start; (p < pages) && (p < start + pp->num_affines); p++) {
+		for (p = start; (p < pages) && (p < start + job->priv->num_affines); p++) {
 			gnome_print_meta_render_data_page (mp, data, len, p, TRUE);
 		}
 		/* Finish multipage */
@@ -770,7 +750,6 @@ gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint pa
 static void
 job_update_layout_data (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
 	ArtDRect area, r;
 	gdouble t;
 	gdouble a[6];
@@ -778,112 +757,107 @@ job_update_layout_data (GnomePrintJob *job)
 
 	g_return_if_fail (job->priv);
 
-	pp = job->priv;
-
 	job_parse_config_data (job);
 
 	/* Now comes the fun part */
 
-	if (pp->num_affines < 1)
+	if (job->priv->num_affines < 1)
 		return;
-	if ((fabs (pp->pw) < EPSILON) || (fabs (pp->ph) < EPSILON))
+	if ((fabs (job->priv->pw) < EPSILON) || (fabs (job->priv->ph) < EPSILON))
 		return;
 
 	/* Initial setup */
 	/* Calculate PP2PA */
 	/* We allow only rectilinear setups, so we can cheat */
-	pp->PP2PA[0] = pp->porient[0];
-	pp->PP2PA[1] = pp->porient[1];
-	pp->PP2PA[2] = pp->porient[2];
-	pp->PP2PA[3] = pp->porient[3];
-	t = pp->pw * pp->PP2PA[0] + pp->ph * pp->PP2PA[2];
-	pp->PP2PA[4] = (t < 0) ? -t : 0.0;
-	t = pp->pw * pp->PP2PA[1] + pp->ph * pp->PP2PA[3];
-	pp->PP2PA[5] = (t < 0) ? -t : 0.0;
-	PRINT_AFFINE ("PP2PA:", &pp->PP2PA[0]);
+	job->priv->PP2PA[0] = job->priv->porient[0];
+	job->priv->PP2PA[1] = job->priv->porient[1];
+	job->priv->PP2PA[2] = job->priv->porient[2];
+	job->priv->PP2PA[3] = job->priv->porient[3];
+	t = job->priv->pw * job->priv->PP2PA[0] + job->priv->ph * job->priv->PP2PA[2];
+	job->priv->PP2PA[4] = (t < 0) ? -t : 0.0;
+	t = job->priv->pw * job->priv->PP2PA[1] + job->priv->ph * job->priv->PP2PA[3];
+	job->priv->PP2PA[5] = (t < 0) ? -t : 0.0;
+	PRINT_AFFINE ("PP2PA:", &job->priv->PP2PA[0]);
 
 	/* PPDP - Physical Page Dimensions in Printer */
 	/* A: PhysicalPage X PhysicalOrientation X TRANSLATE -> Physical Page in Printer */
 	area.x0 = 0.0;
 	area.y0 = 0.0;
-	area.x1 = pp->pw;
-	area.y1 = pp->ph;
-	art_drect_affine_transform (&r, &area, pp->PP2PA);
-	pp->PAW = r.x1 - r.x0;
-	pp->PAH = r.y1 - r.y0;
-	if ((pp->PAW < EPSILON) || (pp->PAH < EPSILON))
+	area.x1 = job->priv->pw;
+	area.y1 = job->priv->ph;
+	art_drect_affine_transform (&r, &area, job->priv->PP2PA);
+	job->priv->PAW = r.x1 - r.x0;
+	job->priv->PAH = r.y1 - r.y0;
+	if ((job->priv->PAW < EPSILON) || (job->priv->PAH < EPSILON))
 		return;
 
 	/* Now we have to find the size of layout page */
 	/* Again, knowing that layouts are rectilinear helps us */
-	art_affine_invert (a, pp->affines);
+	art_affine_invert (a, job->priv->affines);
 	PRINT_AFFINE ("INV LY:", &a[0]);
-	pp->LYW = pp->lyw * fabs (pp->pw * a[0] + pp->ph * a[2]);
-	pp->LYH = pp->lyh * fabs (pp->pw * a[1] + pp->ph * a[3]);
-	PRINT_2 ("LY Dimensions:", pp->LYW, pp->LYH);
+	job->priv->LYW = job->priv->lyw * fabs (job->priv->pw * a[0] + job->priv->ph * a[2]);
+	job->priv->LYH = job->priv->lyh * fabs (job->priv->pw * a[1] + job->priv->ph * a[3]);
+	PRINT_2 ("LY Dimensions:", job->priv->LYW, job->priv->LYH);
 
 	/* Calculate LP2LY */
 	/* We allow only rectilinear setups, so we can cheat */
-	pp->LP2LY[0] = pp->lorient[0];
-	pp->LP2LY[1] = pp->lorient[1];
-	pp->LP2LY[2] = pp->lorient[2];
-	pp->LP2LY[3] = pp->lorient[3];
+	job->priv->LP2LY[0] = job->priv->lorient[0];
+	job->priv->LP2LY[1] = job->priv->lorient[1];
+	job->priv->LP2LY[2] = job->priv->lorient[2];
+	job->priv->LP2LY[3] = job->priv->lorient[3];
 	/* Delay */
-	pp->LP2LY[4] = 0.0;
-	pp->LP2LY[5] = 0.0;
+	job->priv->LP2LY[4] = 0.0;
+	job->priv->LP2LY[5] = 0.0;
 	/* Meanwhile find logical width and height */
 	area.x0 = 0.0;
 	area.y0 = 0.0;
-	area.x1 = pp->LYW;
-	area.y1 = pp->LYH;
-	art_affine_invert (a, pp->LP2LY);
+	area.x1 = job->priv->LYW;
+	area.y1 = job->priv->LYH;
+	art_affine_invert (a, job->priv->LP2LY);
 	art_drect_affine_transform (&r, &area, a);
-	pp->LW = r.x1 - r.x0;
-	pp->LH = r.y1 - r.y0;
-	if ((pp->LW < EPSILON) || (pp->LH < EPSILON))
+	job->priv->LW = r.x1 - r.x0;
+	job->priv->LH = r.y1 - r.y0;
+	if ((job->priv->LW < EPSILON) || (job->priv->LH < EPSILON))
 		return;
-	PRINT_2 ("L Dimensions", pp->LW, pp->LH);
+	PRINT_2 ("L Dimensions", job->priv->LW, job->priv->LH);
 	/* Now complete matrix calculation */
-	t = pp->LW * pp->LP2LY[0] + pp->LH * pp->LP2LY[2];
-	pp->LP2LY[4] = (t < 0) ? -t : 0.0;
-	t = pp->LW * pp->LP2LY[1] + pp->LH * pp->LP2LY[3];
-	pp->LP2LY[5] = (t < 0) ? -t : 0.0;
-	PRINT_AFFINE ("LP2LY:", &pp->LP2LY[0]);
+	t = job->priv->LW * job->priv->LP2LY[0] + job->priv->LH * job->priv->LP2LY[2];
+	job->priv->LP2LY[4] = (t < 0) ? -t : 0.0;
+	t = job->priv->LW * job->priv->LP2LY[1] + job->priv->LH * job->priv->LP2LY[3];
+	job->priv->LP2LY[5] = (t < 0) ? -t : 0.0;
+	PRINT_AFFINE ("LP2LY:", &job->priv->LP2LY[0]);
 
 	/* Good, now generate actual layout matrixes */
 
-	pp->LY_AFFINES = g_new (gdouble, 6 * pp->num_affines);
+	job->priv->LY_AFFINES = g_new (gdouble, 6 * job->priv->num_affines);
 
 	/* Extra fun */
 
-	for (i = 0; i < pp->num_affines; i++) {
+	for (i = 0; i < job->priv->num_affines; i++) {
 		gdouble ly2p[6];
 		gdouble *ly2pa;
 		/* Calculate Layout -> Physical Page affine */
-		memcpy (ly2p, pp->affines + 6 * i, 6 * sizeof (gdouble));
-		ly2p[4] *= pp->pw;
-		ly2p[5] *= pp->ph;
+		memcpy (ly2p, job->priv->affines + 6 * i, 6 * sizeof (gdouble));
+		ly2p[4] *= job->priv->pw;
+		ly2p[5] *= job->priv->ph;
 		/* PRINT_AFFINE ("Layout -> Physical:", &l2p[0]); */
-		art_affine_multiply (pp->LY_AFFINES + 6 * i, pp->LP2LY, ly2p);
+		art_affine_multiply (job->priv->LY_AFFINES + 6 * i, job->priv->LP2LY, ly2p);
 		ly2pa = g_new (gdouble, 6);
-		art_affine_multiply (ly2pa, pp->LY_AFFINES + 6 * i, pp->PP2PA);
-		pp->LY_LIST = g_list_prepend (pp->LY_LIST, ly2pa);
+		art_affine_multiply (ly2pa, job->priv->LY_AFFINES + 6 * i, job->priv->PP2PA);
+		job->priv->LY_LIST = g_list_prepend (job->priv->LY_LIST, ly2pa);
 	}
 
-	pp->LY_LIST = g_list_reverse (pp->LY_LIST);
+	job->priv->LY_LIST = g_list_reverse (job->priv->LY_LIST);
 }
 
 static void
 job_parse_config_data (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
 	const GnomePrintUnit *unit;
 	GPANode *layout;
 
 	g_return_if_fail (job->priv);
 
-	pp = job->priv;
-
 	job_clear_config_data (job);
 
 	g_return_if_fail (job->config);
@@ -892,19 +866,19 @@ job_parse_config_data (GnomePrintJob *job)
 
 
 	/* Physical size */
-	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_WIDTH, &pp->pw, &unit)) {
-		gnome_print_convert_distance (&pp->pw, unit, GNOME_PRINT_PS_UNIT);
+	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_WIDTH, &job->priv->pw, &unit)) {
+		gnome_print_convert_distance (&job->priv->pw, unit, GNOME_PRINT_PS_UNIT);
 	}
-	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &pp->ph, &unit)) {
-		gnome_print_convert_distance (&pp->ph, unit, GNOME_PRINT_PS_UNIT);
+	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &job->priv->ph, &unit)) {
+		gnome_print_convert_distance (&job->priv->ph, unit, GNOME_PRINT_PS_UNIT);
 	}
 	/* Physical orientation */
-	gnome_print_config_get_transform (job->config, GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, pp->porient);
+	gnome_print_config_get_transform (job->config, GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, job->priv->porient);
 	/* Logical orientation */
-	gnome_print_config_get_transform (job->config, GNOME_PRINT_KEY_PAGE_ORIENTATION_MATRIX, pp->lorient);
+	gnome_print_config_get_transform (job->config, GNOME_PRINT_KEY_PAGE_ORIENTATION_MATRIX, job->priv->lorient);
 	/* Layout size */
-	gnome_print_config_get_double (job->config, GNOME_PRINT_KEY_LAYOUT_WIDTH, &pp->lyw);
-	gnome_print_config_get_double (job->config, GNOME_PRINT_KEY_LAYOUT_HEIGHT, &pp->lyh);
+	gnome_print_config_get_double (job->config, GNOME_PRINT_KEY_LAYOUT_WIDTH, &job->priv->lyw);
+	gnome_print_config_get_double (job->config, GNOME_PRINT_KEY_LAYOUT_HEIGHT, &job->priv->lyh);
 
 	/* Now come the affines */
 	layout = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (job->config), GNOME_PRINT_KEY_LAYOUT);
@@ -939,8 +913,8 @@ job_parse_config_data (GnomePrintJob *job)
 				}
 				gpa_node_unref (pages);
 				if (pagenum == numlp) {
-					pp->num_affines = numlp;
-					pp->affines = affines;
+					job->priv->num_affines = numlp;
+					job->priv->affines = affines;
 				} else {
 					g_free (affines);
 				}
@@ -948,9 +922,9 @@ job_parse_config_data (GnomePrintJob *job)
 		}
 		gpa_node_unref (layout);
 	} else {
-		pp->affines = g_new (gdouble, 6);
-		art_affine_identity (pp->affines);
-		pp->num_affines = 1;
+		job->priv->affines = g_new (gdouble, 6);
+		art_affine_identity (job->priv->affines);
+		job->priv->num_affines = 1;
 	}
 }
 
@@ -960,30 +934,26 @@ job_parse_config_data (GnomePrintJob *job)
 static void
 job_clear_config_data (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
-
 	g_return_if_fail (job->priv);
 
-	pp = job->priv;
-
-	pp->pw = A4_WIDTH;
-	pp->ph = A4_HEIGHT;
-	art_affine_identity (pp->porient);
-	art_affine_identity (pp->lorient);
-	pp->lyw = pp->pw;
-	pp->lyh = pp->ph;
-	pp->num_affines = 0;
-	if (pp->affines) {
-		g_free (pp->affines);
-		pp->affines = NULL;
+	job->priv->pw = A4_WIDTH;
+	job->priv->ph = A4_HEIGHT;
+	art_affine_identity (job->priv->porient);
+	art_affine_identity (job->priv->lorient);
+	job->priv->lyw = job->priv->pw;
+	job->priv->lyh = job->priv->ph;
+	job->priv->num_affines = 0;
+	if (job->priv->affines) {
+		g_free (job->priv->affines);
+		job->priv->affines = NULL;
 	}
-	if (pp->LY_AFFINES) {
-		g_free (pp->LY_AFFINES);
-		pp->LY_AFFINES = NULL;
+	if (job->priv->LY_AFFINES) {
+		g_free (job->priv->LY_AFFINES);
+		job->priv->LY_AFFINES = NULL;
 	}
-	while (pp->LY_LIST) {
-		g_free (pp->LY_LIST->data);
-		pp->LY_LIST = g_list_remove (pp->LY_LIST, pp->LY_LIST->data);
+	while (job->priv->LY_LIST) {
+		g_free (job->priv->LY_LIST->data);
+		job->priv->LY_LIST = g_list_remove (job->priv->LY_LIST, job->priv->LY_LIST->data);
 	}
 }
 
