@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * gnome-print-pango.c
  *
@@ -248,6 +249,9 @@ font_from_pango_font (PangoFont *font)
 	
 	if (FcPatternGetDouble (fcfont->font_pattern, FC_SIZE, 0, &size) != FcResultMatch)
 		return NULL;
+	if (FcPatternGetDouble (fcfont->font_pattern, FC_SIZE, 0, &size) != FcResultMatch &&
+	    FcPatternGetDouble (fcfont->font_pattern, FC_PIXEL_SIZE, 0, &size) != FcResultMatch)
+		return NULL;
 
 	return gnome_font_find_from_filename (filename, id, size);
 }
@@ -295,7 +299,7 @@ gnome_print_pango_glyph_string (GnomePrintContext *gpc, PangoFont *font, PangoGl
 			int y = gi->geometry.y_offset;
 			
 			gnome_glyphlist_moveto (glyph_list,
-						(gdouble) x / PANGO_SCALE, (gdouble) y / PANGO_SCALE);
+						(gdouble) x / PANGO_SCALE, (gdouble) -y / PANGO_SCALE);
 			gnome_glyphlist_glyph (glyph_list, gi->glyph);
 		}
 			
