@@ -122,7 +122,7 @@ static gboolean gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFo
 static gboolean gnome_print_ps2_print_bpath (GnomePrintPs2 *ps2, const ArtBpath *bpath);
 
 static gchar *gnome_print_ps2_get_date (void);
-static gboolean gnome_print_ps2_fprintf (GnomePrintPs2 *ps2, const char *format, ...);
+static gboolean gnome_print_ps2_fprintf		 (GnomePrintPs2 *ps2, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
 static gboolean gnome_print_ps2_print_double	 (GnomePrintPs2 *ps2, const gchar *format, gdouble x);
 static gboolean gnome_print_ps2_print_octal_byte (GnomePrintPs2 *ps2, guint8 c);
 
