@@ -99,7 +99,7 @@ static void gnome_print_svg_finalize (GObject *object);
 
 static gint gnome_print_svg_fprintf_double (GnomePrintSvg *svg, 
 					    const gchar *format, gdouble x);
-static gint gnome_print_svg_fprintf (GnomePrintSvg *svg, const char *format, ...);
+static gint gnome_print_svg_fprintf (GnomePrintSvg *svg, const char *format, ...) G_GNUC_PRINTF (2, 3);
 
 /*************************************************************/
 /*************************************************************/
@@ -223,13 +223,13 @@ gnome_print_svg_construct (GnomePrintContext *ctx)
 
 	if (svg->page_width > 0.) {
 		gnome_print_svg_fprintf (svg, " width=\"");
-		gnome_print_svg_fprintf_double (svg, "%g", svg->page_width);
+		gnome_print_svg_fprintf_double (svg, "%f", svg->page_width);
 		gnome_print_svg_fprintf (svg, "\"");
 	}
 
 	if (svg->page_height > 0.) {
 		gnome_print_svg_fprintf (svg, " height=\"");
-		gnome_print_svg_fprintf_double (svg, "%g", svg->page_height);
+		gnome_print_svg_fprintf_double (svg, "%f", svg->page_height);
 		gnome_print_svg_fprintf (svg, "\"");
 	}
 
@@ -283,9 +283,9 @@ gnome_print_svg_print_bpath (GnomePrintSvg *svg, const ArtBpath *bpath)
 			started = closed = FALSE;
 
 			gnome_print_svg_fprintf (svg, " M");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->x3);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->x3);
 			gnome_print_svg_fprintf (svg, ",");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->y3);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->y3);
 			break;
 		case ART_MOVETO:
 			if (started && closed)
@@ -294,29 +294,29 @@ gnome_print_svg_print_bpath (GnomePrintSvg *svg, const ArtBpath *bpath)
 			started = closed = TRUE;
 
 			gnome_print_svg_fprintf (svg, " M");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->x3);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->x3);
 			gnome_print_svg_fprintf (svg, ",");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->y3);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->y3);
 			break;
 		case ART_LINETO:
 			gnome_print_svg_fprintf (svg, " L");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->x3);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->x3);
 			gnome_print_svg_fprintf (svg, ",");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->y3);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->y3);
 			break;
 		case ART_CURVETO:
 			gnome_print_svg_fprintf (svg, " C");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->x1);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->x1);
 			gnome_print_svg_fprintf (svg, ",");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->y1);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->y1);
 			gnome_print_svg_fprintf (svg, " ");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->x2);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->x2);
 			gnome_print_svg_fprintf (svg, ",");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->y2);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->y2);
 			gnome_print_svg_fprintf (svg, " ");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->x3);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->x3);
 			gnome_print_svg_fprintf (svg, ",");
-			gnome_print_svg_fprintf_double (svg, "%g", bpath->y3);
+			gnome_print_svg_fprintf_double (svg, "%f", bpath->y3);
 			break;
 		default:
 			g_warning ("Path structure is corrupted");
@@ -356,17 +356,17 @@ static gint
 gnome_print_svg_matrix (GnomePrintSvg *svg, const gdouble * ctm)
 {
 	gnome_print_svg_fprintf (svg, " transform=\"matrix(");
-	gnome_print_svg_fprintf_double (svg, "%g", ctm[0]);
+	gnome_print_svg_fprintf_double (svg, "%f", ctm[0]);
 	gnome_print_svg_fprintf (svg, ", ");
-	gnome_print_svg_fprintf_double (svg, "%g", ctm[1]);
+	gnome_print_svg_fprintf_double (svg, "%f", ctm[1]);
 	gnome_print_svg_fprintf (svg, ", ");
-	gnome_print_svg_fprintf_double (svg, "%g", ctm[2]);
+	gnome_print_svg_fprintf_double (svg, "%f", ctm[2]);
 	gnome_print_svg_fprintf (svg, ", ");
-	gnome_print_svg_fprintf_double (svg, "%g", ctm[3]);
+	gnome_print_svg_fprintf_double (svg, "%f", ctm[3]);
 	gnome_print_svg_fprintf (svg, ", ");
-	gnome_print_svg_fprintf_double (svg, "%g", ctm[4]);
+	gnome_print_svg_fprintf_double (svg, "%f", ctm[4]);
 	gnome_print_svg_fprintf (svg, ", ");
-	gnome_print_svg_fprintf_double (svg, "%g", ctm[5]);
+	gnome_print_svg_fprintf_double (svg, "%f", ctm[5]);
 	gnome_print_svg_fprintf (svg, ")\"");
 
 	return GNOME_PRINT_OK;
@@ -402,7 +402,7 @@ gnome_print_svg_showpage (GnomePrintContext *pc)
 	return GNOME_PRINT_OK;
 }
 
-/* Note "format" should be locale independent, so it should not use %g */
+/* Note "format" should be locale independent, so it should not use %f */
 /* and friends */
 static gint
 gnome_print_svg_fprintf (GnomePrintSvg *svg, const char *format, ...)
@@ -535,7 +535,7 @@ gnome_print_svg_set_line (GnomePrintSvg *svg)
 	ctx = GNOME_PRINT_CONTEXT (svg);
 
 	gnome_print_svg_fprintf (svg, " stroke-width=\"");
-	gnome_print_svg_fprintf_double (svg, "%g", 
+	gnome_print_svg_fprintf_double (svg, "%f", 
 					   gp_gc_get_linewidth (ctx->gc));
 	gnome_print_svg_fprintf (svg, "\"");
 
@@ -543,7 +543,7 @@ gnome_print_svg_set_line (GnomePrintSvg *svg)
 	gnome_print_svg_fprintf (svg, " stroke-linejoin=\"%s\"", art_linejoin_to_svg(gp_gc_get_linejoin (ctx->gc)));
 
 	gnome_print_svg_fprintf (svg, " stroke-miterlimit=\"");
-	gnome_print_svg_fprintf_double (svg, "%g", 
+	gnome_print_svg_fprintf_double (svg, "%f", 
 					   gp_gc_get_miterlimit (ctx->gc));
 	gnome_print_svg_fprintf (svg, "\"");
 
@@ -564,13 +564,13 @@ gnome_print_svg_set_dash (GnomePrintSvg *svg)
 		gnome_print_svg_fprintf (svg, " stroke-dasharray=\"");
 		for (i = 0; i < dash->n_dash; i++) {
 			gnome_print_svg_fprintf (svg, ",");
-			gnome_print_svg_fprintf_double (svg, "%g", dash->dash[i]);
+			gnome_print_svg_fprintf_double (svg, "%f", dash->dash[i]);
 		}
 		gnome_print_svg_fprintf (svg, "\"");
 	}
 
 	gnome_print_svg_fprintf (svg, " stroke-dashoffset=\"");
-	gnome_print_svg_fprintf_double (svg, "%g", dash->n_dash > 0 ? dash->offset : 0.0);
+	gnome_print_svg_fprintf_double (svg, "%f", dash->n_dash > 0 ? dash->offset : 0.0);
 	gnome_print_svg_fprintf (svg, "\"");
 
 	return GNOME_PRINT_OK;
@@ -580,7 +580,7 @@ static gint
 gnome_print_svg_set_font (GnomePrintSvg *svg, const GnomeFont *gnome_font)
 {
 	gnome_print_svg_fprintf (svg, " font-size=\"");
-	gnome_print_svg_fprintf_double (svg, "%g", gnome_font_get_size(gnome_font));
+	gnome_print_svg_fprintf_double (svg, "%f", gnome_font_get_size(gnome_font));
 	gnome_print_svg_fprintf (svg, "pt\"");
 
 	gnome_print_svg_fprintf (svg, " font-style=\"%s\"",
@@ -729,8 +729,6 @@ static void _png_flush(png_structp png_ptr)
 {
 }
 
-#define EPSILON 1e-9
-
 static gint gnome_print_svg_image (GnomePrintContext *pc, const gdouble *a, 
 				   const guchar *px, gint w, gint h, gint rowstride, gint ch)
 {
@@ -769,7 +767,7 @@ static gint gnome_print_svg_image (GnomePrintContext *pc, const gdouble *a,
 		
 		g_warning("PNG encoding problems\n");
 
-		return GNOME_PRINT_OK;
+		return GNOME_PRINT_ERROR_UNKNOWN;
 	}
 
 	pBB = g_byte_array_new();
@@ -795,9 +793,7 @@ static gint gnome_print_svg_image (GnomePrintContext *pc, const gdouble *a,
 		png_write_rows(png_ptr, &pRow, 1);
 	}
 
-	/*
-	  Wrap things up with libpng
-	*/
+	/* Wrap things up with libpng */
 	png_write_end(png_ptr, info_ptr);
 
 	/* clean up after the write, and free any memory allocated */
@@ -825,9 +821,9 @@ static gint gnome_print_svg_image (GnomePrintContext *pc, const gdouble *a,
 		buffer[1] = '\n';
 
 		gnome_print_svg_fprintf (svg, "<image width=\"");
-		gnome_print_svg_fprintf_double (svg, "%g", w);
+		gnome_print_svg_fprintf_double (svg, "%f", w);
 		gnome_print_svg_fprintf(svg, "\" height=\"");
-		gnome_print_svg_fprintf_double (svg, "%g", h);
+		gnome_print_svg_fprintf_double (svg, "%f", h);
 		gnome_print_svg_fprintf(svg, "\"");
 		gnome_print_svg_matrix (svg, ctm);
 		gnome_print_svg_fprintf (svg, " xlink:href=\"data:image/png;base64,");		
@@ -880,6 +876,8 @@ static gunichar gnome_print_unichar_for_glyph (GnomePrintSvg * svg, GnomeFont *
 
 #undef UCS2_UPPER_BOUND
 
+#define EPSILON 1e-9
+
 static gint gnome_print_svg_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphList *gl)
 {
 	GnomePrintSvg * svg;
