@@ -235,7 +235,7 @@ warn_of_unknown_encoding (char const *encoding)
 }
 
 static char *
-get_paper_text (ppd_file_t *ppd, ppd_size_t *size)
+get_paper_text (ppd_file_t const *ppd, ppd_size_t const *size)
 {
 	/* This is dumb and slow and ugly and crappy and I hate myself. (Dave)*/
 	int i, j, k, len;
@@ -307,7 +307,7 @@ option_list_new_with_default (GPANode      *parent,
 }
 
 static GPANode *
-load_paper_sizes (ppd_file_t *ppd, GPANode *parent)
+load_paper_sizes (ppd_file_t const *ppd, GPANode *parent)
 {
 	ppd_option_t *option;
 	GPANode *node;
