@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: sft.h,v 1.5 2004/12/03 18:32:57 jody Exp $ */
+/* $Id: sft.h,v 1.6 2005/01/04 20:10:46 jody Exp $ */
 /* @(#)sft.h 1.17 03/01/08 SMI */
 
 /*
@@ -152,41 +152,17 @@ extern "C" {
 /*@}*/
 
 #ifndef STSF
-#if 1
 /* glib already deals with different compilers */
 #include <glibconfig.h>
 
-typedef guchar     byte;
-typedef guchar     uint8;
-typedef gchar      int8;
-typedef guint16    uint16;
-typedef gint16     int16;
-
-typedef guint32    uint32;
-typedef gint32     int32;
-typedef guint64    uint64;
-typedef gint64     int64;
-#  else
-/*- XXX These should be dynamically configured */ /*FOLD00*/
-    typedef unsigned char         byte;
-    typedef unsigned char         uint8;
-    typedef signed char           int8;
-    typedef unsigned short int    uint16;
-    typedef short int             int16;
-
-    typedef unsigned int          uint32;
-    typedef int                   int32;
-    typedef unsigned long long    uint64;
-    typedef long long             int64;
-#  endif
 /*@{*/
-    typedef int16       F2Dot14;            /**< fixed: 2.14 */
-    typedef int32       F16Dot16;           /**< fixed: 16.16 */
+    typedef gint16       F2Dot14;            /**< fixed: 2.14 */
+    typedef gint32       F16Dot16;           /**< fixed: 16.16 */
 /*@}*/
 
     typedef struct {
-        uint16 s;
-        uint16 d;
+        guint16 s;
+        guint16 d;
     } uint16pair;
 #endif
 
@@ -275,43 +251,43 @@ typedef gint64     int64;
     /*- In horisontal writing mode right sidebearing is calculated using this formula
      *- rsb = aw - (lsb + xMax - xMin) -*/
      typedef struct {
-         int16  xMin;
-         int16  yMin;
-         int16  xMax;
-         int16  yMax;
-         uint16 aw;                /*- Advance Width (horisontal writing mode)    */
-         int16  lsb;               /*- Left sidebearing (horisontal writing mode) */
-         uint16 ah;                /*- advance height (vertical writing mode)     */
-         int16  tsb;               /*- top sidebearing (vertical writing mode)    */
+         gint16  xMin;
+         gint16  yMin;
+         gint16  xMax;
+         gint16  yMax;
+         guint16 aw;                /*- Advance Width (horisontal writing mode)    */
+         gint16  lsb;               /*- Left sidebearing (horisontal writing mode) */
+         guint16 ah;                /*- advance height (vertical writing mode)     */
+         gint16  tsb;               /*- top sidebearing (vertical writing mode)    */
      } TTGlyphMetrics;
 
 
     /** Structure used by GetTTSimpleGlyphMetrics() and GetTTSimpleCharMetrics() functions */
     typedef struct {
-        uint16 adv;                         /**< advance width or height            */
-        int16 sb;                           /**< left or top sidebearing            */
+        guint16 adv;                         /**< advance width or height            */
+        gint16 sb;                           /**< left or top sidebearing            */
     } TTSimpleGlyphMetrics;
 
     /** Structure returned by ReadGlyphMetrics() */
     typedef struct {
-        uint16 aw, ah;
-        int16 lsb, tsb;
+        guint16 aw, ah;
+        gint16 lsb, tsb;
     } TTFullSimpleGlyphMetrics;
 
 
 /** Structure used by the TrueType Creator and GetRawGlyphData() */
 
     typedef struct {
-        uint32 glyphID;                     /**< glyph ID                           */
-        uint16 nbytes;                      /**< number of bytes in glyph data      */
-        byte  *ptr;                         /**< pointer to glyph data              */
-        uint16 aw;                          /**< advance width                      */
-        int16  lsb;                         /**< left sidebearing                   */
-        uint16 compflag;                    /**< 0- if non-composite, 1- otherwise  */
-        uint16 npoints;                     /**< number of points                   */
-        uint16 ncontours;                   /**< number of contours                 */
+        guint32 glyphID;                     /**< glyph ID                           */
+        guint16 nbytes;                      /**< number of bytes in glyph data      */
+        guint8  *ptr;                         /**< pointer to glyph data              */
+        guint16 aw;                          /**< advance width                      */
+        gint16  lsb;                         /**< left sidebearing                   */
+        guint16 compflag;                    /**< 0- if non-composite, 1- otherwise  */
+        guint16 npoints;                     /**< number of points                   */
+        guint16 ncontours;                   /**< number of contours                 */
         /* */
-        uint32 newID;                       /**< used internally by the TTCR        */
+        guint32 newID;                       /**< used internally by the TTCR        */
     } GlyphData;
 
 
@@ -319,20 +295,20 @@ typedef gint64     int64;
     /* STSF defines NameRecord and FUnitBBox structures in its own include file sttypes.h */
 
     typedef struct {
-        int16 xMin;
-        int16 yMin;
-        int16 xMax;
-        int16 yMax;
+        gint16 xMin;
+        gint16 yMin;
+        gint16 xMax;
+        gint16 yMax;
     } FUnitBBox;
 
 /** Structure used by the TrueType Creator and CreateTTFromTTGlyphs() */
     typedef struct {
-        uint16 platformID;                  /**< Platform ID                                            */
-        uint16 encodingID;                  /**< Platform-specific encoding ID                          */
-        uint16 languageID;                  /**< Language ID                                            */
-        uint16 nameID;                      /**< Name ID                                                */
-        uint16 slen;                        /**< String length in bytes                                 */
-        byte  *sptr;                        /**< Pointer to string data (not zero-terminated!)          */
+        guint16 platformID;                  /**< Platform ID                                            */
+        guint16 encodingID;                  /**< Platform-specific encoding ID                          */
+        guint16 languageID;                  /**< Language ID                                            */
+        guint16 nameID;                      /**< Name ID                                                */
+        guint16 slen;                        /**< String length in bytes                                 */
+        guint8  *sptr;                        /**< Pointer to string data (not zero-terminated!)          */
     } NameRecord;
 #endif
 
@@ -342,14 +318,14 @@ typedef gint64     int64;
 
     typedef struct {
         char *family;             /**< family name                                             */
-        uint16 *ufamily;		  /**< family name UCS2                                         */
+        guint16 *ufamily;		  /**< family name UCS2                                         */
         char *subfamily;          /**< subfamily name                                          */
         char *psname;             /**< PostScript name                                         */
         int   weight;             /**< value of WeightClass or 0 if can't be determined        */
         int   width;              /**< value of WidthClass or 0 if can't be determined         */
         int   pitch;              /**< 0: proportianal font, otherwise: monospaced             */
         int   italicAngle;        /**< in counter-clockwise degrees * 65536                    */
-        uint16 fsSelection;       /**< fsSelection field of OS/2 table                         */
+        guint16 fsSelection;       /**< fsSelection field of OS/2 table                         */
         int   xMin;               /**< global bounding box: xMin                               */
         int   yMin;               /**< global bounding box: yMin                               */
         int   xMax;               /**< global bounding box: xMax                               */
@@ -367,12 +343,12 @@ typedef gint64     int64;
         int   winDescent;         /**< descender metric for Windows                            */
         int   symbolEncoded;      /**< 1: MS symbol encoded 0: not symbol encoded              */
         int   rangeFlag;          /**< if set to 1 Unicode Range flags are applicable          */
-        uint32 ur1;               /**< bits 0 - 31 of Unicode Range flags                      */
-        uint32 ur2;               /**< bits 32 - 63 of Unicode Range flags                     */
-        uint32 ur3;               /**< bits 64 - 95 of Unicode Range flags                     */
-        uint32 ur4;               /**< bits 96 - 127 of Unicode Range flags                    */
-        byte   panose[10];        /**< PANOSE classification number                            */
-        uint16 typeFlags;		  /**< type flags (copyright information)                      */
+        guint32 ur1;               /**< bits 0 - 31 of Unicode Range flags                      */
+        guint32 ur2;               /**< bits 32 - 63 of Unicode Range flags                     */
+        guint32 ur3;               /**< bits 64 - 95 of Unicode Range flags                     */
+        guint32 ur4;               /**< bits 96 - 127 of Unicode Range flags                    */
+        guint8   panose[10];        /**< PANOSE classification number                            */
+        guint16 typeFlags;		  /**< type flags (copyright information)                      */
     } TTGlobalFontInfo;
 
 /** Structure used by KernGlyphs()      */
@@ -384,12 +360,12 @@ typedef gint64     int64;
 
 /** ControlPoint structure used by GetTTGlyphPoints() */
     typedef struct {
-        uint32 flags;             /**< 00000000 00000000 e0000000 bbbbbbbb */
-        /**< b - byte flags from the glyf array  */
+        guint32 flags;             /**< 00000000 00000000 e0000000 bbbbbbbb */
+        /**< b - guint8 flags from the glyf array  */
         /**< e == 0 - regular point              */
         /**< e == 1 - end contour                */
-        int16 x;                  /**< X coordinate in EmSquare units      */
-        int16 y;                  /**< Y coordinate in EmSquare units      */
+        gint16 x;                  /**< X coordinate in EmSquare units      */
+        gint16 y;                  /**< Y coordinate in EmSquare units      */
     } ControlPoint;
 
     typedef struct _TrueTypeFont TrueTypeFont;
@@ -416,7 +392,7 @@ typedef gint64     int64;
  * @return value of SFErrCodes enum
  * @ingroup sft
  */
-    int  OpenTTFont(const char *fname, uint32 facenum, TrueTypeFont**);
+    int  OpenTTFont(const char *fname, guint32 facenum, TrueTypeFont**);
 
 /*
  * TrueTypeFont destructor. Deallocates the memory.
@@ -437,7 +413,7 @@ typedef gint64     int64;
  * @ingroup sft
  *
  */
-int GetTTGlyphPoints(TrueTypeFont *ttf, uint32 glyphID, ControlPoint **pointArray);
+int GetTTGlyphPoints(TrueTypeFont *ttf, guint32 glyphID, ControlPoint **pointArray);
 
 /*
  * Extracts bounding boxes in normalized FUnits (1000/em) for all glyphs in the
@@ -464,7 +440,7 @@ FUnitBBox *GetTTGlyphBoundingBoxes(TrueTypeFont *ttf);
  * @ingroup sft
  *
  */
-    GlyphData *GetTTRawGlyphData(TrueTypeFont *ttf, uint32 glyphID);
+    GlyphData *GetTTRawGlyphData(TrueTypeFont *ttf, guint32 glyphID);
 
 #ifndef NO_LIST
 /*
@@ -482,7 +458,7 @@ FUnitBBox *GetTTGlyphBoundingBoxes(TrueTypeFont *ttf);
  * @ingroup sft
  *
  */
-    int GetTTGlyphComponents(TrueTypeFont *ttf, uint32 glyphID, list glyphlist);
+    int GetTTGlyphComponents(TrueTypeFont *ttf, guint32 glyphID, list glyphlist);
 #endif
 
 /*
@@ -527,7 +503,7 @@ FUnitBBox *GetTTGlyphBoundingBoxes(TrueTypeFont *ttf);
  * @ingroup sft
  *
  */
-    int  CreateT3FromTTGlyphs(TrueTypeFont *ttf, FILE *outf, const char *fname, uint16 *glyphArray, byte *encoding, int nGlyphs, int wmode);
+    int  CreateT3FromTTGlyphs(TrueTypeFont *ttf, FILE *outf, const char *fname, guint16 *glyphArray, guint8 *encoding, int nGlyphs, int wmode);
 #endif
 
 #ifndef NO_TTCR
@@ -553,12 +529,12 @@ FUnitBBox *GetTTGlyphBoundingBoxes(TrueTypeFont *ttf);
  */
     int  CreateTTFromTTGlyphs(TrueTypeFont  *ttf,
                               const char    *fname,
-                              uint16        *glyphArray,
-                              byte          *encoding,
+                              guint16        *glyphArray,
+                              guint8          *encoding,
                               int            nGlyphs,
                               int            nNameRecs,
                               NameRecord    *nr,
-                              uint32        flags);
+                              guint32        flags);
 #endif
 
 #ifndef NO_TYPE42
@@ -585,15 +561,15 @@ FUnitBBox *GetTTGlyphBoundingBoxes(TrueTypeFont *ttf);
     int  CreateT42FromTTGlyphs(TrueTypeFont  *ttf,
                                FILE          *outf,
                                const char    *psname,
-                               uint16        *glyphArray,
-                               byte          *encoding,
+                               guint16        *glyphArray,
+                               guint8          *encoding,
                                int            nGlyphs);
 #endif
 
 /*
  * Queries full glyph metrics for one glyph
  */
-void GetTTGlyphMetrics(TrueTypeFont *ttf, uint32 glyphID, TTGlyphMetrics *metrics);
+void GetTTGlyphMetrics(TrueTypeFont *ttf, guint32 glyphID, TTGlyphMetrics *metrics);
 
 
 /*
@@ -606,7 +582,7 @@ void GetTTGlyphMetrics(TrueTypeFont *ttf, uint32 glyphID, TTGlyphMetrics *metric
  * @ingroup sft
  *
  */
-TTSimpleGlyphMetrics *GetTTSimpleGlyphMetrics(TrueTypeFont *ttf, uint16 *glyphArray, int nGlyphs, int mode);
+TTSimpleGlyphMetrics *GetTTSimpleGlyphMetrics(TrueTypeFont *ttf, guint16 *glyphArray, int nGlyphs, int mode);
 
 #ifndef NO_MAPPERS
 /*
@@ -623,7 +599,7 @@ TTSimpleGlyphMetrics *GetTTSimpleGlyphMetrics(TrueTypeFont *ttf, uint16 *glyphAr
  * @ingroup sft
  *
  */
-    TTSimpleGlyphMetrics *GetTTSimpleCharMetrics(TrueTypeFont *ttf, uint16 firstChar, int nChars, int mode);
+    TTSimpleGlyphMetrics *GetTTSimpleCharMetrics(TrueTypeFont *ttf, guint16 firstChar, int nChars, int mode);
 
 /*
  * Maps a Unicode (UCS-2) string to a glyph array. Returns the number of glyphs in the array,
@@ -637,16 +613,16 @@ TTSimpleGlyphMetrics *GetTTSimpleGlyphMetrics(TrueTypeFont *ttf, uint16 *glyphAr
  * @return MapString() returns -1 if the TrueType font has no usable 'cmap' tables.
  *         Otherwise it returns the number of characters processed: <b>nChars</b>
  *
- * glyphIDs of TrueType fonts are 2 byte positive numbers. glyphID of 0 denotes a missing
+ * glyphIDs of TrueType fonts are 2 guint8 positive numbers. glyphID of 0 denotes a missing
  * glyph and traditionally defaults to an empty square.
- * glyphArray should be at least sizeof(uint16) * nchars bytes long. If glyphArray is NULL
+ * glyphArray should be at least sizeof(guint16) * nchars bytes long. If glyphArray is NULL
  * MapString() replaces the UCS-2 characters in str with glyphIDs.
  * @ingroup sft
  */
 #ifdef USE_GSUB
-    int MapString(TrueTypeFont *ttf, uint16 *str, int nchars, uint16 *glyphArray, int bvertical);
+    int MapString(TrueTypeFont *ttf, guint16 *str, int nchars, guint16 *glyphArray, int bvertical);
 #else
-    int MapString(TrueTypeFont *ttf, uint16 *str, int nchars, uint16 *glyphArray);
+    int MapString(TrueTypeFont *ttf, guint16 *str, int nchars, guint16 *glyphArray);
 #endif
 
 /*
@@ -659,9 +635,9 @@ TTSimpleGlyphMetrics *GetTTSimpleGlyphMetrics(TrueTypeFont *ttf, uint16 *glyphAr
  * @ingroup sft
  */
 #ifdef USE_GSUB
-    uint16 MapChar(TrueTypeFont *ttf, uint16 ch, int bvertical);
+    guint16 MapChar(TrueTypeFont *ttf, guint16 ch, int bvertical);
 #else
-    uint16 MapChar(TrueTypeFont *ttf, uint16 ch);
+    guint16 MapChar(TrueTypeFont *ttf, guint16 ch);
 #endif
 #endif
 
@@ -690,7 +666,7 @@ TTSimpleGlyphMetrics *GetTTSimpleGlyphMetrics(TrueTypeFont *ttf, uint16 *glyphAr
  * @ingroup sft
  *
  */
-void KernGlyphs(TrueTypeFont *ttf, uint16 *glyphs, int nglyphs, int wmode, KernData *kern);
+void KernGlyphs(TrueTypeFont *ttf, guint16 *glyphs, int nglyphs, int wmode, KernData *kern);
 
 /*
  * Returns nonzero if font is a symbol encoded font
@@ -702,18 +678,18 @@ int CheckSymbolEncoding(TrueTypeFont* ttf);
  * DEPRECATED - use ExtractTable instead
  */
 #if 0
-byte *ExtractCmap(TrueTypeFont *ttf);
+guint8 *ExtractCmap(TrueTypeFont *ttf);
 #endif
 
 /*
  * Extracts a table from a font, allocates memort for it and returns a pointer to it.
  */
-byte *ExtractTable(TrueTypeFont *ttf, uint32 tag);
+guint8 *ExtractTable(TrueTypeFont *ttf, guint32 tag);
 
 /*
  * Returns a pointer to the table but does not allocate memory for it.
  */
-const byte *GetTable(TrueTypeFont *ttf, uint32 tag);
+const guint8 *GetTable(TrueTypeFont *ttf, guint32 tag);
 
 /*
  * Functions that do not use TrueTypeFont structure
@@ -736,9 +712,9 @@ const byte *GetTable(TrueTypeFont *ttf, uint32 tag);
  *
  */
 
-TTFullSimpleGlyphMetrics *ReadGlyphMetrics(byte *hmtx, byte *vmtx, int hcount, int vcount, int gcount, int UPEm, uint16 *glyphArray, int nGlyphs);
+TTFullSimpleGlyphMetrics *ReadGlyphMetrics(guint8 *hmtx, guint8 *vmtx, int hcount, int vcount, int gcount, int UPEm, guint16 *glyphArray, int nGlyphs);
 
-void ReadSingleGlyphMetrics(byte *hmtx, byte *vmtx, int hcount, int vcount, int gcount, int UPEm, uint16 glyphID, TTFullSimpleGlyphMetrics *metrics);
+void ReadSingleGlyphMetrics(guint8 *hmtx, guint8 *vmtx, int hcount, int vcount, int gcount, int UPEm, guint16 glyphID, TTFullSimpleGlyphMetrics *metrics);
 
 
 /*
@@ -748,7 +724,7 @@ void ReadSingleGlyphMetrics(byte *hmtx, byte *vmtx, int hcount, int vcount, int
  *
  * @return           number of bytes in it
  */
-uint32 GetKernSubtableLength(byte *kern);
+guint32 GetKernSubtableLength(guint8 *kern);
 
 /*
  *  Kerns a pair of glyphs.
@@ -763,39 +739,39 @@ uint32 GetKernSubtableLength(byte *kern);
  * @param x          X-axis kerning value is returned here
  * @param y          Y-axis kerning value is returned here
  */
-void KernGlyphPair(int kerntype, uint32 nkern, byte **kern, int unitsPerEm, int wmode, uint32 a, uint32 b, int *x, int *y);
+void KernGlyphPair(int kerntype, guint32 nkern, guint8 **kern, int unitsPerEm, int wmode, guint32 a, guint32 b, int *x, int *y);
 
 
 
 /*- private definitions */ /*FOLD00*/
 
 struct _TrueTypeFont {
-    uint32 tag;
+    guint32 tag;
 
     char   *fname;
     off_t  fsize;
-    byte   *ptr;
+    guint8   *ptr;
 
     char   *psname;
     char   *family;
-    uint16  *ufamily;
+    guint16  *ufamily;
     char   *subfamily;
 
-    uint32 ntables;
-    uint32 tdoffset;                              /* offset to the table directory (!= 0 for TrueType collections)      */
-    uint32 *goffsets;
+    guint32 ntables;
+    guint32 tdoffset;                              /* offset to the table directory (!= 0 for TrueType collections)      */
+    guint32 *goffsets;
     int    nglyphs;
     int    unitsPerEm;
     int    numberOfHMetrics;
     int    numOfLongVerMetrics;                   /* if this number is not 0, font has vertical metrics information */
-    byte   *cmap;
+    guint8   *cmap;
     int    cmapType;
-    uint16 (*mapper)(const byte *, uint16);       /* character to glyphID translation function                          */
+    guint16 (*mapper)(const guint8 *, guint16);       /* character to glyphID translation function                          */
     void   **tables;                              /* array of pointers to tables                                        */
-    uint32 *tlens;                                /* array of table lengths                                             */
+    guint32 *tlens;                                /* array of table lengths                                             */
     int    kerntype;                              /* Defined in the KernType enum                                       */
-    uint32 nkern;                                 /* number of kern subtables                                           */
-    byte   **kerntables;                          /* array of pointers to kern subtables                                */
+    guint32 nkern;                                 /* number of kern subtables                                           */
+    guint8   **kerntables;                          /* array of pointers to kern subtables                                */
 #ifdef USE_GSUB
     void   *pGSubstitution;                       /* info provided by GSUB for UseGSUB()                                */
 #endif
